@@ -15,6 +15,7 @@
 #include "gesture-drag.h"
 #include "gesture-swipe.h"
 #include "layer-shell-effects.h"
+#include "touch-point.h"
 
 #define _XOPEN_SOURCE 700
 #include <assert.h>
@@ -855,7 +856,7 @@ phoc_handle_shell_reveal (struct wlr_surface *surface, double lx, double ly, int
 
 
 static void
-phoc_passthrough_cursor (PhocCursor *self, uint32_t time)
+phoc_cursor_do_passthrough (PhocCursor *self, uint32_t time)
 {
   PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   double sx, sy;
@@ -869,7 +870,7 @@ phoc_passthrough_cursor (PhocCursor *self, uint32_t time)
   if (surface)
     client = wl_resource_get_client (surface->resource);
 
-  if (surface && !phoc_seat_allow_input (seat, surface->resource))
+  if (surface && !phoc_seat_is_input_allowed (seat, surface->resource))
     return;
 
   if (priv->image_client != client || !client) {
@@ -1199,94 +1200,110 @@ phoc_cursor_update_focus (PhocCursor *self)
 
   clock_gettime (CLOCK_MONOTONIC, &now);
 
-  phoc_passthrough_cursor (self, timespec_to_msec (&now));
+  phoc_cursor_do_passthrough (self, timespec_to_msec (&now));
 }
 
+static void
+phoc_cursor_do_move (PhocCursor *self, uint32_t time)
+{
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
+  PhocSeat *seat = self->seat;
+  PhocView *view = phoc_seat_get_focus_view (seat);
+  struct wlr_box geom;
+
+  if (!view)
+    return;
+
+  phoc_view_get_geometry (view, &geom);
+  double dx = self->cursor->x - self->offs_x;
+  double dy = self->cursor->y - self->offs_y;
+  PhocOutput *output = phoc_desktop_layout_get_output (desktop, self->cursor->x, self->cursor->y);
+  if (!output)
+    return;
+
+  struct wlr_box output_box;
+  wlr_output_layout_get_box (desktop->layout, output->wlr_output, &output_box);
+
+  bool output_is_landscape = output_box.width > output_box.height;
+
+  if (phoc_view_is_fullscreen (view)) {
+    phoc_view_set_fullscreen (view, true, output);
+  } else if (self->cursor->y < output_box.y + PHOC_EDGE_SNAP_THRESHOLD) {
+    phoc_cursor_suggest_view_state_change (self, view, output, PHOC_VIEW_STATE_MAXIMIZED, -1);
+  } else if (output_is_landscape &&
+             self->cursor->x < output_box.x + PHOC_EDGE_SNAP_THRESHOLD) {
+    phoc_cursor_suggest_view_state_change (self,
+                                           view,
+                                           output,
+                                           PHOC_VIEW_STATE_TILED,
+                                           PHOC_VIEW_TILE_LEFT);
+  } else if (output_is_landscape &&
+             self->cursor->x > output_box.x + output_box.width - PHOC_EDGE_SNAP_THRESHOLD) {
+    phoc_cursor_suggest_view_state_change (self,
+                                           view,
+                                           output,
+                                           PHOC_VIEW_STATE_TILED,
+                                           PHOC_VIEW_TILE_RIGHT);
+  } else {
+    phoc_cursor_clear_view_state_change (self);
+    phoc_view_restore (view);
+    phoc_view_move (view, self->view_x + dx - geom.x * phoc_view_get_scale (view),
+                    self->view_y + dy - geom.y * phoc_view_get_scale (view));
+  }
+}
+
+
+static void
+phoc_cursor_do_resize (PhocCursor *self, uint32_t time)
+{
+  PhocSeat *seat = self->seat;
+  PhocView *view = phoc_seat_get_focus_view (seat);
+  struct wlr_box geom;
+
+  if (!view)
+    return;
+
+  phoc_view_get_geometry (view, &geom);
+  double dx = self->cursor->x - self->offs_x;
+  double dy = self->cursor->y - self->offs_y;
+  double x = view->box.x;
+  double y = view->box.y;
+  int width = self->view_width;
+  int height = self->view_height;
+  if (self->resize_edges & WLR_EDGE_TOP) {
+    y = self->view_y + dy - geom.y * phoc_view_get_scale (view);
+    height -= dy;
+    if (height < 1)
+      y += height;
+  } else if (self->resize_edges & WLR_EDGE_BOTTOM) {
+    height += dy;
+  }
+  if (self->resize_edges & WLR_EDGE_LEFT) {
+    x = self->view_x + dx - geom.x * phoc_view_get_scale (view);
+    width -= dx;
+    if (width < 1)
+      x += width;
+  } else if (self->resize_edges & WLR_EDGE_RIGHT) {
+    width += dx;
+  }
+  phoc_view_move_resize (view, x, y, MAX (1, width), MAX (1, height));
+}
+
+
 void
 phoc_cursor_update_position (PhocCursor *self, uint32_t time)
 {
-  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocCursorPrivate *priv = phoc_cursor_get_instance_private (self);
-  PhocSeat *seat = self->seat;
-  PhocView *view;
 
   switch (priv->mode) {
   case PHOC_CURSOR_PASSTHROUGH:
-    phoc_passthrough_cursor (self, time);
+    phoc_cursor_do_passthrough (self, time);
     break;
   case PHOC_CURSOR_MOVE:
-    view = phoc_seat_get_focus_view (seat);
-    if (view != NULL) {
-      struct wlr_box geom;
-      phoc_view_get_geometry (view, &geom);
-      double dx = self->cursor->x - self->offs_x;
-      double dy = self->cursor->y - self->offs_y;
-      PhocOutput *output = phoc_desktop_layout_get_output (desktop, self->cursor->x, self->cursor->y);
-      if (!output)
-        return;
-
-      struct wlr_box output_box;
-      wlr_output_layout_get_box (desktop->layout, output->wlr_output, &output_box);
-
-      bool output_is_landscape = output_box.width > output_box.height;
-
-      if (phoc_view_is_fullscreen (view)) {
-        phoc_view_set_fullscreen (view, true, output);
-      } else if (self->cursor->y < output_box.y + PHOC_EDGE_SNAP_THRESHOLD) {
-        phoc_cursor_suggest_view_state_change (self, view, output, PHOC_VIEW_STATE_MAXIMIZED, -1);
-      } else if (output_is_landscape &&
-                 self->cursor->x < output_box.x + PHOC_EDGE_SNAP_THRESHOLD) {
-        phoc_cursor_suggest_view_state_change (self,
-                                               view,
-                                               output,
-                                               PHOC_VIEW_STATE_TILED,
-                                               PHOC_VIEW_TILE_LEFT);
-      } else if (output_is_landscape &&
-                 self->cursor->x > output_box.x + output_box.width - PHOC_EDGE_SNAP_THRESHOLD) {
-        phoc_cursor_suggest_view_state_change (self,
-                                               view,
-                                               output,
-                                               PHOC_VIEW_STATE_TILED,
-                                               PHOC_VIEW_TILE_RIGHT);
-      } else {
-        phoc_cursor_clear_view_state_change (self);
-        phoc_view_restore (view);
-        phoc_view_move (view, self->view_x + dx - geom.x * phoc_view_get_scale (view),
-                        self->view_y + dy - geom.y * phoc_view_get_scale (view));
-      }
-    }
+    phoc_cursor_do_move (self, time);
     break;
   case PHOC_CURSOR_RESIZE:
-    view = phoc_seat_get_focus_view (seat);
-    if (view != NULL) {
-      struct wlr_box geom;
-      phoc_view_get_geometry (view, &geom);
-      double dx = self->cursor->x - self->offs_x;
-      double dy = self->cursor->y - self->offs_y;
-      double x = view->box.x;
-      double y = view->box.y;
-      int width = self->view_width;
-      int height = self->view_height;
-      if (self->resize_edges & WLR_EDGE_TOP) {
-        y = self->view_y + dy - geom.y * phoc_view_get_scale (view);
-        height -= dy;
-        if (height < 1) {
-          y += height;
-        }
-      } else if (self->resize_edges & WLR_EDGE_BOTTOM) {
-        height += dy;
-      }
-      if (self->resize_edges & WLR_EDGE_LEFT) {
-        x = self->view_x + dx - geom.x * phoc_view_get_scale (view);
-        width -= dx;
-        if (width < 1) {
-          x += width;
-        }
-      } else if (self->resize_edges & WLR_EDGE_RIGHT) {
-        width += dx;
-      }
-      phoc_view_move_resize (view, x, y, MAX (1, width), MAX (1, height));
-    }
+    phoc_cursor_do_resize (self, time);
     break;
   default:
     g_error ("Invalid cursor mode %d", priv->mode);
@@ -1556,7 +1573,7 @@ phoc_cursor_handle_touch_down (PhocCursor                  *self,
   struct wlr_surface *surface = phoc_desktop_wlr_surface_at (desktop, lx, ly, &sx, &sy, &view);
   bool shell_revealed = phoc_handle_shell_reveal (surface, lx, ly, PHOC_SHELL_REVEAL_TOUCH_THRESHOLD);
 
-  if (!shell_revealed && surface && phoc_seat_allow_input (seat, surface->resource)) {
+  if (!shell_revealed && surface && phoc_seat_is_input_allowed (seat, surface->resource)) {
     struct wlr_surface *root = wlr_surface_get_root_surface (surface);
 
     send_touch_down (seat, surface, event, sx, sy);
@@ -1710,7 +1727,7 @@ phoc_cursor_handle_touch_motion (PhocCursor                    *self,
       }
     }
 
-    if (phoc_seat_allow_input (self->seat, surface->resource))
+    if (phoc_seat_is_input_allowed (self->seat, surface->resource))
       send_touch_motion (self->seat, surface, event, sx, sy);
   }
 
