@@ -11,8 +11,11 @@
 #include <wlr/types/wlr_alpha_modifier_v1.h>
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_cursor.h>
+#include <wlr/types/wlr_cursor_shape_v1.h>
 #include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_export_dmabuf_v1.h>
+#include <wlr/types/wlr_ext_image_capture_source_v1.h>
+#include <wlr/types/wlr_ext_image_copy_capture_v1.h>
 #include <wlr/types/wlr_fractional_scale_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
 #include <wlr/types/wlr_idle_notify_v1.h>
@@ -42,16 +45,19 @@
 #include "output.h"
 #include "seat.h"
 #include "server.h"
+#include "shortcuts-inhibit.h"
 #include "color-rect.h"
 #include "timed-animation.h"
-#include "utils.h"
+#include "outputs-states.h"
 #include "view.h"
 #include "virtual.h"
 #include "xdg-activation-v1.h"
 #include "wlr-layer-shell-unstable-v1-protocol.h"
-#include "gesture-swipe.h"
 #include "layer-shell-effects.h"
-#include "xdg-surface.h"
+#include "workspace.h"
+#include "workspace-indicator.h"
+#include "workspace-manager.h"
+#include "xdg-toplevel.h"
 #include "xdg-toplevel-decoration.h"
 #include "xwayland-surface.h"
 
@@ -60,6 +66,7 @@
 #define PHOC_EXT_FOREIGN_TOPLEVEL_LIST_VERSION 1
 #define PHOC_XDG_SHELL_VERSION 6
 #define PHOC_LAYER_SHELL_VERSION 3
+#define PHOC_PRESENTATION_TIME_VERSION 2
 
 #define PHOC_ANIM_ALWAYS_ON_TOP_DURATION  300
 #define PHOC_ANIM_ALWAYS_ON_TOP_COLOR_ON  (PhocColor){0.5f, 0.0f, 0.3f, 0.5f}
@@ -81,8 +88,6 @@ static GParamSpec *props[PROP_LAST_PROP];
 
 
 typedef struct _PhocDesktopPrivate {
-  GQueue                *views;
-
   PhocIdleInhibit       *idle_inhibit;
 
   gboolean               enable_animations;
@@ -90,11 +95,15 @@ typedef struct _PhocDesktopPrivate {
   GSettings             *settings;
   GSettings             *interface_settings;
 
+  PhocOutputsStates     *outputs_states;
+
   /* Protocols from wlroots */
   struct wlr_data_control_manager_v1 *data_control_manager_v1;
+  struct wlr_ext_image_copy_capture_manager_v1 *ext_image_copy_capture_manager_v1;
   struct wlr_idle_notifier_v1 *idle_notifier_v1;
   struct wlr_screencopy_manager_v1 *screencopy_manager_v1;
   struct wl_listener gamma_control_set_gamma;
+  struct wl_listener request_set_cursor_shape;
 
   /* Protocols without upstreamable implementations */
   PhocPhoshPrivate      *phosh;
@@ -103,6 +112,9 @@ typedef struct _PhocDesktopPrivate {
   /* Protocols that should go upstream */
   PhocLayerShellEffects *layer_shell_effects;
   PhocDeviceState       *device_state;
+
+  PhocWorkspaceManager  *workspace_manager;
+  PhocWorkspace         *active_workspace;
 } PhocDesktopPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (PhocDesktop, phoc_desktop, G_TYPE_OBJECT);
@@ -188,7 +200,9 @@ desktop_view_at (PhocDesktop         *self,
                  double              *sx,
                  double              *sy)
 {
-  for (GList *l = phoc_desktop_get_views (self)->head; l; l = l->next) {
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+
+  for (GList *l = phoc_workspace_get_views (priv->active_workspace)->head; l; l = l->next) {
     PhocView *view = PHOC_VIEW (l->data);
 
     if (phoc_desktop_view_check_visibility (self, view) && view_at (view, lx, ly, surface, sx, sy))
@@ -336,7 +350,10 @@ phoc_desktop_view_check_visibility (PhocDesktop *self, PhocView *view)
     goto out;
   }
 
-  g_assert_true (priv->views->head);
+  if (!phoc_workspace_has_view (priv->active_workspace, view)) {
+    visible = FALSE;
+    goto out;
+  }
 
   /* current heuristics work well only for single output */
   if (wl_list_length (&self->outputs) != 1)
@@ -356,7 +373,7 @@ phoc_desktop_view_check_visibility (PhocDesktop *self, PhocView *view)
   if (!self->maximize)
     goto out;
 
-  top_view = phoc_desktop_get_view_by_index (self, 0);
+  top_view = phoc_workspace_get_view_by_index (priv->active_workspace, 0);
   /* XWayland parent relations can be complicated and aren't described by PhocView
    * relationships very well at the moment, so just make all XWayland windows visible
    * when some XWayland window is active for now */
@@ -378,11 +395,32 @@ phoc_desktop_view_check_visibility (PhocDesktop *self, PhocView *view)
   return visible;
 }
 
+
+struct move_to_layout_space_data {
+  double center_x;
+  double center_y;
+};
+
+
+static gboolean
+move_to_layout_space_iter (PhocDesktop *self, PhocView *view, gpointer user_data)
+{
+  struct move_to_layout_space_data *data = user_data;
+  struct wlr_box box;
+
+  phoc_view_get_box (view, &box);
+  if (wlr_output_layout_intersects (self->layout, NULL, &box))
+    return TRUE;
+
+  phoc_view_move (view, data->center_x - box.width / 2, data->center_y - box.height / 2);
+  return TRUE;
+}
+
+
 static void
 handle_layout_change (struct wl_listener *listener, void *data)
 {
   PhocDesktop *self;
-  PhocDesktopPrivate *priv;
   struct wlr_output *center_output;
   struct wlr_box center_output_box;
   double center_x, center_y;
@@ -393,26 +431,21 @@ handle_layout_change (struct wl_listener *listener, void *data)
   if (center_output == NULL)
     return;
 
-  priv = phoc_desktop_get_instance_private (self);
   wlr_output_layout_get_box (self->layout, center_output, &center_output_box);
   center_x = center_output_box.x + center_output_box.width / 2;
   center_y = center_output_box.y + center_output_box.height / 2;
 
   /* Make sure all views are on an existing output */
-  for (GList *l = priv->views->head; l; l = l->next) {
-    PhocView *view = PHOC_VIEW (l->data);
-    struct wlr_box box;
-
-    phoc_view_get_box (view, &box);
-    if (wlr_output_layout_intersects (self->layout, NULL, &box))
-      continue;
-
-    phoc_view_move (view, center_x - box.width / 2, center_y - box.height / 2);
-  }
+  phoc_desktop_for_each_view (self,
+                              move_to_layout_space_iter,
+                               (gpointer)&(struct move_to_layout_space_data) {
+                                 .center_x = center_x,
+                                 .center_y = center_y,
+                               });
 
   /* Damage all outputs since the move above damaged old layout space */
-  wl_list_for_each(output, &self->outputs, link)
-    phoc_output_damage_whole(output);
+  wl_list_for_each (output, &self->outputs, link)
+    phoc_output_damage_whole (output);
 }
 
 
@@ -527,6 +560,7 @@ on_output_destroyed (PhocDesktop *self, PhocOutput *destroyed_output)
   g_object_unref (destroyed_output);
 }
 
+
 static void
 handle_new_output (struct wl_listener *listener, void *data)
 {
@@ -546,6 +580,26 @@ handle_new_output (struct wl_listener *listener, void *data)
 }
 
 
+static void
+handle_request_set_cursor_shape (struct wl_listener *listener, void *data)
+{
+  const struct wlr_cursor_shape_manager_v1_request_set_shape_event *event = data;
+  PhocSeat *seat = PHOC_SEAT (event->seat_client->seat->data);
+  struct wlr_surface *focused_surface = seat->seat->pointer_state.focused_surface;
+  struct wl_client *focused_client = NULL;
+
+  if (focused_surface)
+    focused_client = wl_resource_get_client (focused_surface->resource);
+
+  if (focused_client == NULL || event->seat_client->client != focused_client) {
+    g_debug ("Denying request to set cursor shape from unfocused client");
+    return;
+  }
+
+  phoc_cursor_set_name (seat->cursor, focused_client, wlr_cursor_shape_v1_name (event->shape));
+}
+
+
 static void
 handle_backend_destroy (struct wl_listener *listener, void *data)
 {
@@ -564,11 +618,10 @@ phoc_desktop_constructed (GObject *object)
   PhocServer *server = phoc_server_get_default ();
   struct wl_display *wl_display = phoc_server_get_wl_display (server);
   struct wlr_backend *wlr_backend = phoc_server_get_backend (server);
+  struct wlr_cursor_shape_manager_v1 *cursor_shape_manager;
 
   G_OBJECT_CLASS (phoc_desktop_parent_class)->constructed (object);
 
-  wl_list_init (&self->outputs);
-
   self->new_output.notify = handle_new_output;
   wl_signal_add (&wlr_backend->events.new_output, &self->new_output);
 
@@ -580,12 +633,12 @@ phoc_desktop_constructed (GObject *object)
   self->layout_change.notify = handle_layout_change;
   wl_signal_add (&self->layout->events.change, &self->layout_change);
 
-  self->xdg_shell = wlr_xdg_shell_create(wl_display, PHOC_XDG_SHELL_VERSION);
+  self->xdg_shell = wlr_xdg_shell_create (wl_display, PHOC_XDG_SHELL_VERSION);
   wl_signal_add (&self->xdg_shell->events.new_toplevel, &self->xdg_shell_toplevel);
   self->xdg_shell_toplevel.notify = phoc_handle_xdg_shell_toplevel;
 
   self->layer_shell = wlr_layer_shell_v1_create (wl_display, PHOC_LAYER_SHELL_VERSION);
-  wl_signal_add(&self->layer_shell->events.new_surface, &self->layer_shell_surface);
+  wl_signal_add (&self->layer_shell->events.new_surface, &self->layer_shell_surface);
   self->layer_shell_surface.notify = phoc_handle_layer_shell_surface;
   priv->layer_shell_effects = phoc_layer_shell_effects_new ();
 
@@ -629,11 +682,20 @@ phoc_desktop_constructed (GObject *object)
                  &self->virtual_keyboard_new);
   self->virtual_keyboard_new.notify = phoc_handle_virtual_keyboard;
 
+  self->keyboard_shortcuts_inhibit = wlr_keyboard_shortcuts_inhibit_v1_create (wl_display);
+  self->keyboard_shortcuts_inhibit_new_inhibitor.notify =
+    phoc_handle_keyboard_shortcuts_inhibit_new_inhibitor;
+  wl_signal_add (&self->keyboard_shortcuts_inhibit->events.new_inhibitor,
+                 &self->keyboard_shortcuts_inhibit_new_inhibitor);
+
   self->virtual_pointer = wlr_virtual_pointer_manager_v1_create (wl_display);
   wl_signal_add (&self->virtual_pointer->events.new_virtual_pointer, &self->virtual_pointer_new);
   self->virtual_pointer_new.notify = phoc_handle_virtual_pointer;
 
   priv->screencopy_manager_v1 = wlr_screencopy_manager_v1_create (wl_display);
+  priv->ext_image_copy_capture_manager_v1 =
+    wlr_ext_image_copy_capture_manager_v1_create (wl_display, 1);
+  wlr_ext_output_image_capture_source_manager_v1_create (wl_display, 1);
 
   self->xdg_decoration_manager = wlr_xdg_decoration_manager_v1_create (wl_display);
   wl_signal_add (&self->xdg_decoration_manager->events.new_toplevel_decoration,
@@ -652,8 +714,12 @@ phoc_desktop_constructed (GObject *object)
   self->pointer_constraint.notify = handle_pointer_constraint;
   wl_signal_add (&self->pointer_constraints->events.new_constraint, &self->pointer_constraint);
 
+  cursor_shape_manager = wlr_cursor_shape_manager_v1_create (wl_display, 1);
+  priv->request_set_cursor_shape.notify = handle_request_set_cursor_shape;
+  wl_signal_add (&cursor_shape_manager->events.request_set_shape, &priv->request_set_cursor_shape);
+
   wlr_alpha_modifier_v1_create (wl_display);
-  wlr_presentation_create (wl_display, wlr_backend);
+  wlr_presentation_create (wl_display, wlr_backend, PHOC_PRESENTATION_TIME_VERSION);
   self->foreign_toplevel_manager_v1 = wlr_foreign_toplevel_manager_v1_create (wl_display);
   self->ext_foreign_toplevel_list_v1 =
     wlr_ext_foreign_toplevel_list_v1_create (wl_display, PHOC_EXT_FOREIGN_TOPLEVEL_LIST_VERSION);
@@ -698,14 +764,14 @@ phoc_desktop_finalize (GObject *object)
   PhocDesktop *self = PHOC_DESKTOP (object);
   PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
 
-  g_clear_pointer (&priv->views, g_queue_free);
-
+  wl_list_remove (&priv->request_set_cursor_shape.link);
   wl_list_remove (&priv->gamma_control_set_gamma.link);
   wl_list_remove (&self->layout_change.link);
   wl_list_remove (&self->xdg_shell_toplevel.link);
   wl_list_remove (&self->layer_shell_surface.link);
   wl_list_remove (&self->xdg_toplevel_decoration.link);
   wl_list_remove (&self->virtual_keyboard_new.link);
+  wl_list_remove (&self->keyboard_shortcuts_inhibit_new_inhibitor.link);
   wl_list_remove (&self->virtual_pointer_new.link);
   wl_list_remove (&self->pointer_constraint.link);
   wl_list_remove (&self->output_manager_apply.link);
@@ -722,6 +788,7 @@ phoc_desktop_finalize (GObject *object)
   g_clear_object (&priv->device_state);
   g_clear_pointer (&self->layout, wlr_output_layout_destroy);
 
+  g_clear_object (&priv->outputs_states);
   g_hash_table_remove_all (self->input_output_map);
   g_hash_table_unref (self->input_output_map);
 
@@ -757,19 +824,98 @@ phoc_desktop_class_init (PhocDesktopClass *klass)
 }
 
 
+static gboolean
+workspace_damage_view_iter (PhocWorkspace *workspace, PhocView *view, gpointer user_data)
+{
+  phoc_view_damage_whole (view);
+  return TRUE;
+}
+
+#define INDICATOR_OFFSET 16
+#define INDICATOR_SIZE 64
+
+static void
+show_workspace_indicator (PhocDesktop *self, int num)
+{
+  PhocOutput *output;
+  g_autoptr (PhocTimedAnimation) fade_anim = NULL;
+  g_autoptr (PhocPropertyEaser) easer = NULL;
+  g_autoptr (PhocColorRect) rect = NULL;
+
+  if (!phoc_desktop_get_enable_animations (self))
+    return;
+
+  wl_list_for_each (output, &self->outputs, link) {
+    GSList *blings = phoc_output_get_blings (output);
+
+    for (GSList *l = blings; l; l = l->next) {
+      if (PHOC_IS_WORKSPACE_INDICATOR (l->data)) {
+        phoc_output_remove_bling (output, l->data);
+        break;
+      }
+    }
+  }
+
+  wl_list_for_each (output, &self->outputs, link) {
+    g_autoptr (PhocWorkspaceIndicator) indicator = NULL;
+
+    indicator = phoc_workspace_indicator_new (PHOC_ANIMATABLE (output),
+                                              num,
+                                              output->lx + INDICATOR_OFFSET,
+                                              output->ly + INDICATOR_OFFSET,
+                                              INDICATOR_SIZE);
+    phoc_output_add_bling (output, PHOC_BLING (indicator));
+    phoc_bling_map (PHOC_BLING (indicator));
+  }
+}
+
+
+static void
+on_active_workspace_changed (PhocDesktop *self, GParamSpec *pspec, PhocWorkspaceManager *manager)
+{
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+  int index;
+
+  if (priv->active_workspace)
+    phoc_workspace_for_each_view (priv->active_workspace, workspace_damage_view_iter, NULL);
+
+  priv->active_workspace = phoc_workspace_manager_get_active (priv->workspace_manager);
+  index = phoc_workspace_manager_get_active_index (priv->workspace_manager);
+  show_workspace_indicator (self, index + 1);
+
+  phoc_workspace_for_each_view (priv->active_workspace, workspace_damage_view_iter, NULL);
+}
+
+
 static void
 phoc_desktop_init (PhocDesktop *self)
 {
   PhocDesktopPrivate *priv;
+  g_autoptr (GError) err = NULL;
+  gboolean success;
+
+  wl_list_init (&self->outputs);
 
   priv = phoc_desktop_get_instance_private (self);
-  priv->views = g_queue_new ();
   priv->enable_animations = TRUE;
 
   self->input_output_map = g_hash_table_new_full (g_str_hash,
                                                   g_str_equal,
                                                   g_free,
                                                   NULL);
+
+  priv->outputs_states = phoc_outputs_states_new (NULL);
+  success = phoc_outputs_states_load (priv->outputs_states, &err);
+  if (!success)
+    g_debug ("Failed to load output states: %s", err->message);
+
+  priv->workspace_manager = phoc_workspace_manager_new ();
+  g_signal_connect_object (priv->workspace_manager,
+                           "notify::active",
+                           G_CALLBACK (on_active_workspace_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+  on_active_workspace_changed (self, NULL, priv->workspace_manager);
 }
 
 
@@ -779,6 +925,29 @@ phoc_desktop_new (void)
   return g_object_new (PHOC_TYPE_DESKTOP, NULL);
 }
 
+
+struct toggle_auto_max_data {
+  PhocInput   *input;
+  gboolean     enable;
+};
+
+
+static gboolean
+toggle_auto_max_iterator (PhocDesktop *self, PhocView *view, gpointer user_data)
+{
+  struct toggle_auto_max_data *data = user_data;
+
+  if (data->enable) {
+    phoc_view_auto_maximize (view);
+    phoc_view_appear_activated (view, true);
+  } else {
+    /* Disabling auto-maximize leaves all views in their current position */
+    phoc_view_appear_activated (view, phoc_input_view_has_focus (data->input, view));
+  }
+
+  return TRUE;
+}
+
 /**
  * phoc_desktop_set_auto_maximize:
  *
@@ -787,8 +956,8 @@ phoc_desktop_new (void)
 void
 phoc_desktop_set_auto_maximize (PhocDesktop *self, gboolean enable)
 {
-  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
   PhocServer *server = phoc_server_get_default();
+  PhocInput *input = phoc_server_get_input (server);
 
   if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_AUTO_MAXIMIZE))) {
     if (enable == FALSE)
@@ -799,24 +968,12 @@ phoc_desktop_set_auto_maximize (PhocDesktop *self, gboolean enable)
   g_debug ("auto-maximize: %d", enable);
   self->maximize = enable;
 
-  /* Disabling auto-maximize leaves all views in their current position */
-  if (!enable) {
-    PhocInput *input = phoc_server_get_input (server);
-
-    for (GList *l = priv->views->head; l; l = l->next) {
-      PhocView *view = PHOC_VIEW (l->data);
-
-      phoc_view_appear_activated (view, phoc_input_view_has_focus (input, view));
-    }
-    return;
-  }
-
-  for (GList *l = priv->views->head; l; l = l->next) {
-    PhocView *view = PHOC_VIEW (l->data);
-
-    phoc_view_auto_maximize (view);
-    phoc_view_appear_activated (view, true);
-  }
+  phoc_desktop_for_each_view (self,
+                              toggle_auto_max_iterator,
+                              (gpointer)&(struct toggle_auto_max_data) {
+                                .input = input,
+                                .enable = enable,
+                              });
 }
 
 gboolean
@@ -1128,40 +1285,21 @@ phoc_desktop_is_privileged_protocol (PhocDesktop *self, const struct wl_global *
     global == self->ext_foreign_toplevel_list_v1->global ||
     global == priv->screencopy_manager_v1->global ||
     global == self->export_dmabuf_manager_v1->global ||
+    global == priv->ext_image_copy_capture_manager_v1->global ||
     global == self->foreign_toplevel_manager_v1->global ||
     global == self->gamma_control_manager_v1->global ||
     global == self->input_method->global ||
+    global == self->keyboard_shortcuts_inhibit->global ||
     global == self->layer_shell->global ||
     global == self->output_manager_v1->global ||
     global == self->output_power_manager_v1->global ||
     global == self->security_context_manager_v1->global ||
     global == self->virtual_keyboard->global ||
-    global == self->virtual_pointer->global
-    );
+    global == self->virtual_pointer->global);
 
   return is_priv;
 }
 
-/**
- * phoc_desktop_get_views:
- * @self: the desktop
- *
- * Get the current views. Don't manipulate the queue directly. This is
- * only meant for reading.
- *
- * Returns:(transfer none): The views.
- */
-GQueue *
-phoc_desktop_get_views (PhocDesktop *self)
-{
-  PhocDesktopPrivate *priv;
-
-  g_assert (PHOC_IS_DESKTOP (self));
-  priv = phoc_desktop_get_instance_private (self);
-
-  return priv->views;
-}
-
 /**
  * phoc_desktop_move_view_to_top:
  * @self: the desktop
@@ -1174,71 +1312,33 @@ phoc_desktop_get_views (PhocDesktop *self)
 void
 phoc_desktop_move_view_to_top (PhocDesktop *self, PhocView *view)
 {
-  GList *view_link;
-  PhocDesktopPrivate *priv;
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+  guint n_workspaces;
 
   g_assert (PHOC_IS_DESKTOP (self));
-  priv = phoc_desktop_get_instance_private (self);
 
-  view_link = g_queue_find (priv->views, view);
-  g_assert (view_link);
-
-  g_queue_unlink (priv->views, view_link);
-
-  if (G_UNLIKELY (phoc_view_is_always_on_top (view))) {
-    g_queue_push_head_link (priv->views, view_link);
-  } else {
-    GList *l = NULL;
-
-    for (l = phoc_desktop_get_views (self)->head; l; l = l->next) {
-      if (!phoc_view_is_always_on_top (PHOC_VIEW (l->data)))
-        break;
-    }
-
-    g_queue_insert_before_link (priv->views, l, view_link);
+  /* Fast path: check active workspace */
+  if (phoc_workspace_has_view (priv->active_workspace, view)) {
+      phoc_workspace_move_view_to_top (priv->active_workspace, view);
+      return;
   }
 
-  phoc_view_damage_whole (view);
-}
+  n_workspaces = phoc_workspace_manager_get_n_workspaces (priv->workspace_manager);
+  for (guint i = 0; i < n_workspaces; i++) {
+    PhocWorkspace *workspace = phoc_workspace_manager_get_by_index (priv->workspace_manager, i);
 
-/**
- * phoc_desktop_has_views:
- * @self: the desktop
- *
- * Check whether the desktop has any views.
- *
- * Returns: %TRUE if there's at least on view, otherwise %FALSE
- */
-gboolean
-phoc_desktop_has_views (PhocDesktop *self)
-{
-  PhocDesktopPrivate *priv;
+    /* Already checked above */
+    if (workspace == priv->active_workspace)
+      continue;
 
-  g_assert (PHOC_IS_DESKTOP (self));
-  priv = phoc_desktop_get_instance_private (self);
+    if (phoc_workspace_has_view (workspace, view)) {
+      phoc_workspace_move_view_to_top (workspace, view);
+      phoc_workspace_manager_set_active (priv->workspace_manager, workspace);
+      return;
+    }
+  }
 
-  return !!priv->views->head;
-}
-
-/**
- * phoc_desktop_get_view_by_index:
- * @self: the desktop
- * @index: the index to get the view for
- *
- * Gets the view at the given position in the queue. If the view is
- * not part of that desktop %NULL is returned.
- *
- * Returns:(transfer none)(nullable): the looked up view
- */
-PhocView *
-phoc_desktop_get_view_by_index (PhocDesktop *self, guint index)
-{
-  PhocDesktopPrivate *priv;
-
-  g_assert (PHOC_IS_DESKTOP (self));
-  priv = phoc_desktop_get_instance_private (self);
-
-  return g_queue_peek_nth (priv->views, index);
+  g_assert_not_reached ();
 }
 
 /**
@@ -1257,8 +1357,7 @@ phoc_desktop_insert_view (PhocDesktop *self, PhocView *view)
   g_assert (PHOC_IS_DESKTOP (self));
   priv = phoc_desktop_get_instance_private (self);
 
-  g_queue_push_head (priv->views, view);
-  phoc_desktop_move_view_to_top (self, view);
+  phoc_workspace_insert_view (priv->active_workspace, view);
 }
 
 /**
@@ -1274,11 +1373,51 @@ gboolean
 phoc_desktop_remove_view (PhocDesktop *self, PhocView *view)
 {
   PhocDesktopPrivate *priv;
+  guint n_workspaces;
 
   g_assert (PHOC_IS_DESKTOP (self));
   priv = phoc_desktop_get_instance_private (self);
+  n_workspaces = phoc_workspace_manager_get_n_workspaces (priv->workspace_manager);
 
-  return g_queue_remove (priv->views, view);
+  for (guint i = 0; i < n_workspaces; i++) {
+    PhocWorkspace *workspace = phoc_workspace_manager_get_by_index (priv->workspace_manager, i);
+
+    if (phoc_workspace_remove_view (workspace, view))
+      return TRUE;
+  }
+
+  return FALSE;
+}
+
+/**
+ * phoc_desktop_for_each_view:
+ * @self: The desktop
+ * @view_iter:(scope call): The iterator
+ * @user_data: The user data
+ *
+ * Invokes `view_iter` on all views passing in `user_data`.
+ */
+void
+phoc_desktop_for_each_view (PhocDesktop *self, PhocDesktopViewIter view_iter, gpointer user_data)
+{
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+  guint n_workspaces;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  n_workspaces = phoc_workspace_manager_get_n_workspaces (priv->workspace_manager);
+  for (guint i = 0; i < n_workspaces; i++) {
+    PhocWorkspace *workspace = phoc_workspace_manager_get_by_index (priv->workspace_manager, i);
+
+    for (GList *l = phoc_workspace_get_views (workspace)->head; l; l = l->next) {
+      PhocView *view = PHOC_VIEW (l->data);
+      gboolean cont;
+
+      cont = (*view_iter)(self, view, user_data);
+      if (!cont)
+        return;
+    }
+  }
 }
 
 
@@ -1363,3 +1502,168 @@ phoc_desktop_set_view_always_on_top (PhocDesktop *self, PhocView *view, gboolean
   wl_list_for_each_reverse (child, &view->stack, parent_link)
     phoc_desktop_set_view_always_on_top (self, child, on_top);
 }
+
+
+static int
+cmp_output_name (gconstpointer a, gconstpointer b)
+{
+  const PhocOutputConfig *config1 = *((PhocOutputConfig **) a);
+  const PhocOutputConfig *config2 = *((PhocOutputConfig **) b);
+
+  return g_ascii_strcasecmp (config1->name, config2->name);
+}
+
+
+static char *
+build_identifier (GPtrArray *output_configs)
+{
+  GString *identifier = g_string_new (NULL);
+
+  for (int i = 0; i < output_configs->len; i++) {
+    PhocOutputConfig *oc = g_ptr_array_index (output_configs, i);
+
+    if (i != 0)
+      g_string_append_c (identifier, '%');
+
+    g_string_append (identifier, oc->name);
+  }
+
+  return g_string_free (identifier, FALSE);
+}
+
+
+static void
+on_outputs_states_save_ready (GObject *object, GAsyncResult *res, gpointer data)
+{
+  g_autoptr (GError) err = NULL;
+  gboolean success;
+
+  success = phoc_outputs_states_save_finish (PHOC_OUTPUTS_STATES (object), res, &err);
+  if (!success)
+    g_critical ("Failed so save output states: %s", err->message);
+}
+
+/**
+ * phoc_desktop_save_outputs_state:
+ * @self: The desktop
+ * @output_configs: (transfer full)(element-type PhocOutputConfig): The output configs to save
+ *
+ * Save the passed in output configuration state.
+ */
+void
+phoc_desktop_save_outputs_state (PhocDesktop *self, GPtrArray *output_configs)
+{
+  PhocServer *server = phoc_server_get_default ();
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+  g_autofree char *identifier = NULL;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  /* Sort so identifier doesn't depend on the order of outputs */
+  g_ptr_array_sort (output_configs, cmp_output_name);
+  identifier = build_identifier (output_configs);
+
+  phoc_outputs_states_update (priv->outputs_states,
+                              identifier,
+                              g_steal_pointer (&output_configs));
+
+  if (phoc_server_get_debug_flags (server) & PHOC_SERVER_DEBUG_FLAG_IGNORE_STATES)
+    return;
+
+  phoc_outputs_states_save_async (priv->outputs_states,
+                                  on_outputs_states_save_ready,
+                                  NULL,
+                                  NULL);
+}
+
+/**
+ * phoc_desktop_get_saved_outputs_state:
+ * @self: The desktop
+ * @output_identifier: The output's identifier
+ *
+ * Get the current target output configuration based on the currently
+ * known outputs.
+ *
+ * Returns: (transfer none): The output config
+ */
+PhocOutputConfig *
+phoc_desktop_get_saved_outputs_state (PhocDesktop *self, const char *output_identifier)
+{
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+  g_autoptr (GPtrArray) current_configs = NULL;
+  GPtrArray *output_configs;
+  PhocOutput *output;
+  g_autofree char *state_identifier = NULL;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  g_debug ("Looking for output state: %s", output_identifier);
+
+  /* Build a stub output configuration to got get an identifier */
+  current_configs = g_ptr_array_new_full (5, (GDestroyNotify) phoc_output_config_destroy);
+  wl_list_for_each (output, &self->outputs, link) {
+    const char *identifier = phoc_output_get_identifier (output);
+    PhocOutputConfig *oc;
+
+    oc = phoc_output_config_new (identifier);
+    g_ptr_array_add (current_configs, oc);
+  }
+  /* Sort so identifier doesn't depend on the order of outputs */
+  g_ptr_array_sort (current_configs, cmp_output_name);
+  state_identifier = build_identifier (current_configs);
+
+  /* Lookup the state of all outputs */
+  output_configs = phoc_outputs_states_lookup (priv->outputs_states, state_identifier);
+  if (!output_configs)
+    return NULL;
+
+  /* TODO: Handle multiple outputs by reconfiguring existing ones too */
+  if (output_configs->len != 1)
+    return NULL;
+
+  /* Get the config for the passed in identifier */
+  for (int i = 0; i < output_configs->len; i++) {
+    PhocOutputConfig *oc = g_ptr_array_index (output_configs, i);
+
+    if (g_strcmp0 (output_identifier, oc->name) == 0)
+      return oc;
+  }
+
+  return NULL;
+}
+
+/**
+ * phoc_desktop_get_workspace_manager:
+ * @self: the desktop
+
+ * Get the workspace manager
+ *
+ * Returns:(transfer none): The workspace manager
+ */
+PhocWorkspaceManager *
+phoc_desktop_get_workspace_manager (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  return priv->workspace_manager;
+}
+
+/**
+ * phoc_desktop_get_active_workspace:
+ * @self: the desktop
+
+ * Get the workspace manager
+ *
+ * Returns:(transfer none): The workspace manager
+ */
+PhocWorkspace *
+phoc_desktop_get_active_workspace (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  return priv->active_workspace;
+}
