@@ -8,9 +8,10 @@
 
 #include "output.h"
 
-#include <wlr/types/wlr_layer_shell_v1.h>
 #include <glib-object.h>
 
+#include <wayland-util.h>
+
 G_BEGIN_DECLS
 
 #define PHOC_TYPE_LAYER_SURFACE (phoc_layer_surface_get_type ())
