@@ -34,6 +34,13 @@ typedef enum _PhocOutputScaleFilter {
 } PhocOutputScaleFilter;
 
 
+typedef enum _PhocOutputAdaptiveSync {
+  PHOC_OUTPUT_ADAPTIVE_SYNC_NONE = 0,
+  PHOC_OUTPUT_ADAPTIVE_SYNC_ENABLED = 1,
+  PHOC_OUTPUT_ADAPTIVE_SYNC_DISABLED = 2,
+} PhocOutputAdaptiveSync;
+
+
 typedef struct {
   gint64            when;
   pixman_region32_t region;
@@ -148,6 +155,8 @@ void        phoc_output_damage_from_surface (PhocOutput         *self,
                                              double              ox,
                                              double              oy,
                                              gboolean            whole);
+gboolean    phoc_output_damage_box (PhocOutput *self, const struct wlr_box *box);
+gboolean    phoc_output_damage_region (PhocOutput *self, const pixman_region32_t *region);
 
 void        phoc_output_update_shell_reveal (PhocOutput *self);
 void        phoc_output_force_shell_reveal (PhocOutput *self, gboolean force);
@@ -156,7 +165,6 @@ gboolean    phoc_output_is_match (PhocOutput *self,
                                   const char *make,
                                   const char *model,
                                   const char *serial);
-gboolean    phoc_output_has_fullscreen_view (PhocOutput *self);
 gboolean    phoc_output_has_layer (PhocOutput *self, enum zwlr_layer_shell_v1_layer layer);
 gboolean    phoc_output_has_shell_revealed (PhocOutput *self);
 
@@ -175,6 +183,7 @@ void       phoc_output_lower_shield          (PhocOutput *self, PhocEasing easin
 void       phoc_output_raise_shield          (PhocOutput *self, gboolean show_spinner);
 float      phoc_output_get_scale             (PhocOutput *self);
 const char *phoc_output_get_name             (PhocOutput *self);
+const char *phoc_output_get_identifier       (PhocOutput *self);
 void       phoc_output_transform_damage      (PhocOutput *self, pixman_region32_t *damage);
 void       phoc_output_transform_box         (PhocOutput *self, struct wlr_box *box);
 GSList    *phoc_output_get_debug_damage      (PhocOutput *self);
@@ -182,4 +191,8 @@ GSList    *phoc_output_get_debug_damage      (PhocOutput *self);
 enum wlr_scale_filter_mode
            phoc_output_get_texture_filter_mode (PhocOutput *self);
 
+void       phoc_output_add_bling (PhocOutput *self, PhocBling *bling);
+void       phoc_output_remove_bling (PhocOutput *self, PhocBling *bling);
+GSList *   phoc_output_get_blings (PhocOutput *self);
+
 G_END_DECLS
