@@ -12,14 +12,17 @@
 #define G_LOG_DOMAIN "phoc-render"
 
 #include "phoc-config.h"
+#include "phoc-tracing.h"
+
 #include "bling.h"
 #include "cursor.h"
 #include "input.h"
-#include "layer-shell.h"
+#include "layer-surface.h"
 #include "seat.h"
 #include "server.h"
 #include "render.h"
 #include "render-private.h"
+#include "touch-point.h"
 #include "xwayland-surface.h"
 #include "utils.h"
 
@@ -148,9 +151,8 @@ render_texture (PhocOutput               *output,
   if (!phoc_utils_is_damaged (&proj_box, ctx->damage, clip_box, &damage))
     goto buffer_damage_finish;
 
-  phoc_output_transform_box (output, &proj_box);
-  phoc_output_transform_damage (output, &damage);
-  transform = wlr_output_transform_compose (surface_transform, output->wlr_output->transform);
+  transform = wlr_output_transform_compose (wlr_output_transform_invert (surface_transform),
+                                            output->wlr_output->transform);
 
   wlr_render_pass_add_texture (ctx->render_pass, &(struct wlr_render_texture_options) {
       .texture = texture,
@@ -191,8 +193,11 @@ render_surface_iterator (PhocOutput         *output,
 
   phoc_utils_scale_box (&dst_box, scale);
   phoc_utils_scale_box (&dst_box, wlr_output->scale);
+  phoc_output_transform_box (output, &dst_box);
+
   phoc_utils_scale_box (&clip_box, scale);
   phoc_utils_scale_box (&clip_box, wlr_output->scale);
+  phoc_output_transform_box (output, &clip_box);
 
   alpha_modifier_state = wlr_alpha_modifier_v1_get_surface_state (surface);
   if (alpha_modifier_state)
@@ -264,6 +269,23 @@ render_layer (enum zwlr_layer_shell_v1_layer layer, PhocRenderContext *ctx)
 }
 
 
+static void
+render_output_blings (PhocOutput *output, PhocRenderContext *ctx)
+{
+  GSList *blings;
+
+  blings = phoc_output_get_blings (output);
+  if (!blings)
+    return;
+
+  for (GSList *l = blings; l; l = l->next) {
+    PhocBling *bling = PHOC_BLING (l->data);
+
+    phoc_bling_render (bling, ctx);
+  }
+}
+
+
 static void
 render_drag_icons (PhocInput *input, PhocRenderContext *ctx)
 {
@@ -328,7 +350,7 @@ view_render_to_buffer_iterator (struct wlr_surface *surface, int sx, int sy, voi
       .texture = texture,
       .src_box = src_box,
       .dst_box = dst_box,
-      .transform = surface->current.transform,
+      .transform = wlr_output_transform_invert (surface->current.transform),
       .alpha = &alpha,
     });
 }
@@ -356,16 +378,23 @@ phoc_renderer_render_view_to_buffer (PhocRenderer      *self,
 
   width = shm_buffer->width;
   height = shm_buffer->height;
-  wlr_drm_format_set_add (&fmt_set, DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_INVALID);
+  wlr_drm_format_set_add (&fmt_set, DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR);
   fmt = wlr_drm_format_set_get (&fmt_set, DRM_FORMAT_ARGB8888);
 
   buffer = wlr_allocator_create_buffer (self->wlr_allocator, width, height, fmt);
   if (!buffer) {
     wlr_drm_format_set_finish (&fmt_set);
-    g_return_val_if_reached (false);
+    g_warning ("Failed to allocate buffer");
+    return false;
   }
 
   render_pass = wlr_renderer_begin_buffer_pass (self->wlr_renderer, buffer, NULL);
+  if (!render_pass) {
+    wlr_drm_format_set_finish (&fmt_set);
+    g_warning ("Failed to start render pass");
+    return false;
+  }
+
   wlr_render_pass_add_rect (render_pass, &(struct wlr_render_rect_options){
       .color = { 0, 0, 0, 0 },
       .blend_mode = WLR_RENDER_BLEND_MODE_NONE,
@@ -419,7 +448,6 @@ render_damage (PhocRenderer *self, PhocRenderContext *ctx)
 
     pixman_region32_init (&clip);
     pixman_region32_copy (&clip, &damage->region);
-    phoc_output_transform_damage (ctx->output, &clip);
 
     /* Using an empty box makes us clip the damage from the whole output buffer */
     wlr_render_pass_add_rect (ctx->render_pass, &(struct wlr_render_rect_options){
@@ -445,33 +473,29 @@ render_damage (PhocRenderer *self, PhocRenderContext *ctx)
 void
 phoc_renderer_render_output (PhocRenderer *self, PhocOutput *output, PhocRenderContext *ctx)
 {
+  gint64 begin_time_nsec G_GNUC_UNUSED = PHOC_TRACE_CURRENT_TIME;
   PhocServer *server = phoc_server_get_default ();
   struct wlr_output *wlr_output = output->wlr_output;
   PhocDesktop *desktop = PHOC_DESKTOP (output->desktop);
+  PhocWorkspace *workspace = phoc_desktop_get_active_workspace (desktop);
   pixman_region32_t *damage = ctx->damage;
-  pixman_region32_t transformed_damage;
 
   g_assert (PHOC_IS_RENDERER (self));
 
-  pixman_region32_init (&transformed_damage);
-
   if (!pixman_region32_not_empty (damage)) {
     /* Output isn't damaged but needs buffer swap */
     goto renderer_end;
   }
 
-  pixman_region32_copy (&transformed_damage, damage);
-  phoc_output_transform_damage (output, &transformed_damage);
-
   wlr_render_pass_add_rect (ctx->render_pass,
                             &(struct wlr_render_rect_options){
                               .box = { .width = wlr_output->width, .height = wlr_output->height },
                               .color = COLOR_BLACK,
-                              .clip = &transformed_damage,
+                              .clip = damage,
                             });
 
   /* If a view is fullscreen on this output, render it */
-  if (output->fullscreen_view != NULL) {
+  if (output->fullscreen_view && phoc_workspace_has_view (workspace, output->fullscreen_view)) {
     PhocView *view = output->fullscreen_view;
 
     render_view (output, view, ctx);
@@ -500,7 +524,7 @@ phoc_renderer_render_output (PhocRenderer *self, PhocOutput *output, PhocRenderC
     render_layer (ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM, ctx);
 
     /* Render all views */
-    for (GList *l = phoc_desktop_get_views (desktop)->tail; l; l = l->prev) {
+    for (GList *l = phoc_workspace_get_views (workspace)->tail; l; l = l->prev) {
       PhocView *view = PHOC_VIEW (l->data);
 
       if (phoc_desktop_view_check_visibility (desktop, view))
@@ -513,8 +537,9 @@ phoc_renderer_render_output (PhocRenderer *self, PhocOutput *output, PhocRenderC
 
   render_layer (ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY, ctx);
 
+  render_output_blings (output, ctx);
+
  renderer_end:
-  pixman_region32_fini (&transformed_damage);
   wlr_output_add_software_cursors_to_render_pass (wlr_output, ctx->render_pass, damage);
 
   if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_TOUCH_POINTS)))
@@ -523,6 +548,10 @@ phoc_renderer_render_output (PhocRenderer *self, PhocOutput *output, PhocRenderC
   g_signal_emit (self, signals[RENDER_END], 0, ctx);
   if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_DAMAGE_TRACKING)))
     render_damage (self, ctx);
+
+  phoc_trace_mark (begin_time_nsec, PHOC_TRACE_CURRENT_TIME - begin_time_nsec,
+                   "phoc", __func__,
+                   "Render output %s", output->wlr_output->name);
 }
 
 
