@@ -1370,16 +1370,13 @@ phoc_seat_view_from_view (PhocSeat *seat, PhocView *view)
 
 
 bool
-phoc_seat_allow_input (PhocSeat *seat, struct wl_resource *resource)
+phoc_seat_is_input_allowed (PhocSeat *seat, struct wl_resource *resource)
 {
   PhocSeatPrivate *priv;
 
   g_assert (PHOC_IS_SEAT (seat));
   priv = phoc_seat_get_instance_private (seat);
 
-  if (!phoc_server_get_allow_input (phoc_server_get_default ()))
-    return false;
-
   return !priv->exclusive_client || wl_resource_get_client (resource) == priv->exclusive_client;
 }
 
@@ -1419,7 +1416,7 @@ phoc_seat_set_focus_view (PhocSeat *seat, PhocView *view)
   priv = phoc_seat_get_instance_private (seat);
 
   g_debug ("Trying to focus view %p", view);
-  if (view && !phoc_seat_allow_input (seat, view->wlr_surface->resource))
+  if (view && !phoc_seat_is_input_allowed (seat, view->wlr_surface->resource))
     return;
 
   /* Make sure the view will be rendered on top of others, even if it's
@@ -1530,14 +1527,17 @@ phoc_seat_set_focus_view (PhocSeat *seat, PhocView *view)
   phoc_input_method_relay_set_focus (&seat->im_relay, view->wlr_surface);
 }
 
-/*
+/**
+ * phoc_seat_set_focus_layer:
+ * @seat: The seat
+ * @layer:(nullable): The layer surface to focus
+ *
  * Focus semantics of layer surfaces are somewhat detached from the normal focus
  * flow. For layers above the shell layer, for example, you cannot unfocus them.
  * You also cannot alt-tab between layer surfaces and shell surfaces.
  */
 void
-phoc_seat_set_focus_layer (PhocSeat                    *seat,
-                           struct wlr_layer_surface_v1 *layer)
+phoc_seat_set_focus_layer (PhocSeat *seat, struct wlr_layer_surface_v1 *layer)
 {
   PhocSeatPrivate *priv;
 
@@ -1565,7 +1565,7 @@ phoc_seat_set_focus_layer (PhocSeat                    *seat,
 
   struct wlr_keyboard *keyboard = wlr_seat_get_keyboard (seat->seat);
 
-  if (!phoc_seat_allow_input (seat, layer->resource))
+  if (!phoc_seat_is_input_allowed (seat, layer->resource))
     return;
 
   if (priv->has_focus) {
@@ -2121,3 +2121,4 @@ phoc_seat_add_shortcuts_inhibit (PhocSeat                                   *sel
                             G_CALLBACK (on_shortcuts_inhibit_destroy),
                             self);
 }
+
