@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2019 Purism SPC
- *               2023-2025 The Phosh Developers
+ *               2023-2024 The Phosh Developers
+ *               2025 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  * Author: Guido Günther <agx@sigxcpu.org>
@@ -26,6 +27,7 @@
 #include <wlr/xwayland/shell.h>
 
 #include <errno.h>
+#include <sys/resource.h>
 
 /* Maximum protocol versions we support */
 #define PHOC_WL_DISPLAY_VERSION 6
@@ -51,6 +53,7 @@ typedef struct _PhocServer {
 
   gboolean             inited;
   gboolean             show_spinner;
+  gboolean             allow_input;
 
   PhocInput           *input;
   PhocConfig          *config;
@@ -68,6 +71,8 @@ typedef struct _PhocServer {
 
   GStrv                dt_compatibles;
 
+  struct rlimit        saved_nofile_rlimit;
+
   struct wl_display   *wl_display;
   guint                wl_source;
 
@@ -154,7 +159,7 @@ phoc_wayland_init (PhocServer *self)
 static void
 on_session_exit (GPid pid, gint status, PhocServer *self)
 {
-  g_autoptr(GError) err = NULL;
+  g_autoptr (GError) err = NULL;
 
   g_return_if_fail (PHOC_IS_SERVER (self));
   g_spawn_close_pid (pid);
@@ -172,21 +177,33 @@ on_session_exit (GPid pid, gint status, PhocServer *self)
 
 
 static void
-on_child_setup (gpointer unused)
+on_child_setup (gpointer data)
 {
+  PhocServer *self = PHOC_SERVER (data);
   sigset_t mask;
 
+  g_assert (PHOC_IS_SERVER (data));
+
   /* phoc wants SIGUSR1 blocked due to wlroots/xwayland but we
      don't want to inherit that to children */
-  sigemptyset(&mask);
-  sigaddset(&mask, SIGUSR1);
-  sigprocmask(SIG_UNBLOCK, &mask, NULL);
+  sigemptyset (&mask);
+  sigaddset (&mask, SIGUSR1);
+  sigprocmask (SIG_UNBLOCK, &mask, NULL);
+
+  /* Restore nofile rlimit */
+  if (self->saved_nofile_rlimit.rlim_cur) {
+    if (setrlimit (RLIMIT_NOFILE, &self->saved_nofile_rlimit)) {
+      g_critical ("Failed to restore nofile rlimit: %s", g_strerror (errno));
+      return;
+    }
+  }
 }
 
 
-static gboolean
-phoc_startup_session_in_idle (PhocServer *self)
+static void
+phoc_startup_session_in_idle (gpointer data)
 {
+  PhocServer *self = PHOC_SERVER (data);
   GPid pid;
   g_auto (GStrv) argv;
   g_autoptr (GError) err = NULL;
@@ -206,7 +223,6 @@ phoc_startup_session_in_idle (PhocServer *self)
     g_critical ("Failed to launch session: %s", err->message);
     g_main_loop_quit (self->mainloop);
   }
-  return FALSE;
 }
 
 
@@ -215,11 +231,35 @@ phoc_startup_session (PhocServer *server)
 {
   gint id;
 
-  id = g_idle_add ((GSourceFunc) phoc_startup_session_in_idle, server);
+  id = g_idle_add_once (phoc_startup_session_in_idle, server);
   g_source_set_name_by_id (id, "[phoc] phoc_startup_session");
 }
 
 
+static void
+phoc_server_raise_nofile_rlimit (PhocServer *self)
+{
+  struct rlimit new_rlimit;
+
+  if (getrlimit (RLIMIT_NOFILE, &self->saved_nofile_rlimit)) {
+    g_critical ("Failed to get nofile rlimit: %s", g_strerror (errno));
+    return;
+  }
+
+  new_rlimit = self->saved_nofile_rlimit;
+  new_rlimit.rlim_cur = new_rlimit.rlim_max;
+
+  if (setrlimit (RLIMIT_NOFILE, &new_rlimit)) {
+    g_critical ("Failed to raise nofile rlimit: %s, max open files is %" G_GUINT64_FORMAT,
+                g_strerror (errno),
+                (guint64)self->saved_nofile_rlimit.rlim_cur);
+    return;
+  }
+
+  g_debug ("Updated nofile current rlimit to %" G_GUINT64_FORMAT, (guint64)new_rlimit.rlim_cur);
+}
+
+
 static void
 on_shell_state_changed (PhocServer *self, GParamSpec *pspec, PhocPhoshPrivate *phosh)
 {
@@ -239,13 +279,14 @@ on_shell_state_changed (PhocServer *self, GParamSpec *pspec, PhocPhoshPrivate *p
       phoc_output_lower_shield (output, PHOC_EASING_EASE_IN_CUBIC, 0);
     /* don't show session init spinner again */
     self->show_spinner = FALSE;
+    self->allow_input = TRUE;
     break;
   case PHOC_PHOSH_PRIVATE_SHELL_STATE_UNKNOWN:
   default:
     /* Shell is gone, raise shields */
-    /* TODO: prevent input without a shell attached */
     wl_list_for_each (output, &self->desktop->outputs, link)
       phoc_output_raise_shield (output, self->show_spinner);
+    self->allow_input = FALSE;
   }
 }
 
@@ -322,9 +363,8 @@ phoc_server_initable_init (GInitable    *initable,
   }
 
   self->renderer = phoc_renderer_new (self->backend, error);
-  if (self->renderer == NULL) {
+  if (self->renderer == NULL)
     return FALSE;
-  }
   wlr_renderer = phoc_renderer_get_wlr_renderer (self->renderer);
   wlr_renderer_init_wl_shm (wlr_renderer, self->wl_display);
 
@@ -339,7 +379,9 @@ phoc_server_initable_init (GInitable    *initable,
 
   self->data_device_manager = wlr_data_device_manager_create (self->wl_display);
 
-  self->compositor = wlr_compositor_create (self->wl_display, PHOC_WL_DISPLAY_VERSION, wlr_renderer);
+  self->compositor = wlr_compositor_create (self->wl_display,
+                                            PHOC_WL_DISPLAY_VERSION,
+                                            wlr_renderer);
   wl_signal_add (&self->compositor->events.new_surface, &self->new_surface);
   self->new_surface.notify = handle_new_surface;
 
@@ -435,7 +477,7 @@ phoc_server_finalize (GObject *object)
   g_clear_pointer (&self->session_exec, g_free);
 
   if (self->inited) {
-    g_unsetenv("WAYLAND_DISPLAY");
+    g_unsetenv ("WAYLAND_DISPLAY");
     self->inited = FALSE;
   }
 
@@ -515,7 +557,7 @@ phoc_server_get_default (void)
 
   if (G_UNLIKELY (instance == NULL)) {
     g_autoptr (GError) err = NULL;
-    g_debug("Creating server");
+    g_debug ("Creating server");
     instance = g_initable_new (PHOC_TYPE_SERVER, NULL, &err, NULL);
     if (instance == NULL) {
       g_critical ("Failed to create server: %s", err->message);
@@ -548,6 +590,8 @@ phoc_server_setup (PhocServer      *self,
                    GMainLoop       *mainloop,
                    PhocServerFlags  flags)
 {
+  const char *socket = NULL;
+
   g_assert (!self->inited);
 
   self->config = config;
@@ -557,9 +601,15 @@ phoc_server_setup (PhocServer      *self,
   self->input = phoc_input_new ();
   self->session_exec = g_strdup (exec);
 
-  const char *socket = wl_display_add_socket_auto (self->wl_display);
+  if (config->socket) {
+    if (wl_display_add_socket (self->wl_display, config->socket) == 0)
+      socket = config->socket;
+  } else {
+    socket = wl_display_add_socket_auto (self->wl_display);
+  }
+
   if (!socket) {
-    g_warning("Unable to open wayland socket: %s", strerror(errno));
+    g_warning ("Unable to open wayland socket: %s", strerror (errno));
     wlr_backend_destroy (self->backend);
     return FALSE;
   }
@@ -567,13 +617,13 @@ phoc_server_setup (PhocServer      *self,
   g_print ("Running compositor on wayland display '%s'\n", socket);
 
   if (!wlr_backend_start (self->backend)) {
-    g_warning("Failed to start backend");
+    g_warning ("Failed to start backend");
     wlr_backend_destroy (self->backend);
     wl_display_destroy (self->wl_display);
     return FALSE;
   }
 
-  g_setenv("WAYLAND_DISPLAY", socket, true);
+  g_setenv ("WAYLAND_DISPLAY", socket, true);
 
   if (self->flags & PHOC_SERVER_FLAG_SHELL_MODE) {
     g_message ("Enabling shell mode");
@@ -582,9 +632,14 @@ phoc_server_setup (PhocServer      *self,
                              G_CALLBACK (on_shell_state_changed),
                              self, G_CONNECT_SWAPPED);
     on_shell_state_changed (self, NULL, phoc_desktop_get_phosh_private (self->desktop));
+  } else {
+    self->allow_input = TRUE;
   }
 
   phoc_wayland_init (self);
+
+  phoc_server_raise_nofile_rlimit (self);
+
   if (self->session_exec)
     phoc_startup_session (self);
 
@@ -645,7 +700,7 @@ phoc_server_get_renderer (PhocServer *self)
  * phoc_server_get_desktop:
  * @self: The server
  *
- * Get's the desktop singleton
+ * Gets the desktop singleton
  *
  * Returns: (transfer none): The desktop
  */
@@ -785,7 +840,7 @@ phoc_server_get_log_domains (PhocServer *self)
  * phoc_server_get_last_active_seat:
  * @self: The server
  *
- * Get's the last active seat.
+ * Gets the last active seat.
  *
  * Returns: (transfer none): The last active seat.
  */
@@ -872,3 +927,12 @@ phoc_server_set_linux_dmabuf_surface_feedback (PhocServer *self,
     wlr_linux_dmabuf_v1_set_surface_feedback (self->linux_dmabuf_v1, view->wlr_surface, NULL);
   }
 }
+
+
+gboolean
+phoc_server_get_allow_input (PhocServer *self)
+{
+  g_assert (PHOC_IS_SERVER (self));
+
+  return self->allow_input;
+}
